@@ -14,21 +14,18 @@
 #    under the License.
 
 from neutron_lib.api.definitions import provider_net as pnet
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
 from neutron_lib import exceptions as n_exc
-
-from midonet.neutron._i18n import _LE, _LW
-from midonet.neutron.client import base as c_base
-from midonet.neutron.common import config  # noqa
-from midonet.neutron.common import constants as m_const
-from midonet.neutron.db import l3_db_midonet
-from midonet.neutron import extensions
-from midonet.neutron.extensions import routerinterfacefip
+from neutron_lib.plugins import constants as plugin_constants
+from oslo_config import cfg
+from oslo_log import helpers as log_helpers
+from oslo_log import log as logging
+from oslo_utils import excutils
 
 from neutron.api import extensions as neutron_extensions
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
 from neutron.db import api as db_api
 from neutron.db import common_db_mixin
 from neutron.db import extraroute_db
@@ -36,21 +33,23 @@ from neutron.db import extraroute_db
 from neutron.db import l3_dvr_db  # noqa
 from neutron.extensions import l3
 from neutron.extensions import multiprovidernet as mpnet
-from oslo_config import cfg
-from oslo_log import helpers as log_helpers
-from oslo_log import log as logging
-from oslo_utils import excutils
+
+from midonet.neutron.client import base as c_base
+from midonet.neutron.common import config  # noqa
+from midonet.neutron.common import constants as m_const
+from midonet.neutron.db import l3_db_midonet
+from midonet.neutron import extensions
+from midonet.neutron.extensions import routerinterfacefip
 
 LOG = logging.getLogger(__name__)
 
 
+@registry.has_registry_receivers
 class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
                              extraroute_db.ExtraRoute_db_mixin,
                              l3_db_midonet.MidonetL3DBMixin):
 
-    """
-    Implements L3 Router service plugin for Midonet.
-    """
+    """Implements L3 Router service plugin for Midonet."""
 
     supported_extension_aliases = ["router", "extraroute", "ext-gw-mode",
                                    "router-interface-fip", "fip64"]
@@ -60,7 +59,6 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
 
     def __init__(self):
         super(MidonetL3ServicePlugin, self).__init__()
-        self.__subscribe()
 
         # Instantiate MidoNet API client
         self.client = c_base.load_client(cfg.CONF.MIDONET)
@@ -71,7 +69,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
 
     @classmethod
     def get_plugin_type(cls):
-        return n_const.L3
+        return plugin_constants.L3
 
     def get_plugin_description(self):
         """Returns string description of the plugin."""
@@ -94,7 +92,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         for seg in self._segments(network):
             if seg[pnet.NETWORK_TYPE] in our_types:
                 return
-        LOG.warning(_LW("Incompatible network %s"), network)
+        LOG.warning("Incompatible network %s", network)
         raise n_exc.BadRequest(resource='router', msg='Incompatible network')
 
     def _validate_router_gw_network(self, context, r):
@@ -105,7 +103,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def create_router(self, context, router):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call create_port inside
             # of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -118,18 +116,18 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
             self.client.create_router_postcommit(r)
         except Exception as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to create a router %(r_id)s in Midonet:"
-                              "%(err)s"), {"r_id": r["id"], "err": ex})
+                LOG.error("Failed to create a router %(r_id)s in Midonet:"
+                          "%(err)s", {"r_id": r["id"], "err": ex})
                 try:
                     self.delete_router(context, r['id'])
                 except Exception:
-                    LOG.exception(_LE("Failed to delete a router %s"), r["id"])
+                    LOG.exception("Failed to delete a router %s", r["id"])
         return r
 
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def update_router(self, context, id, router):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): Updating external_gateway_info causes
             # create_port/delete_port.  This should not call them inside of
             # a transaction.
@@ -143,19 +141,18 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
             self.client.update_router_postcommit(id, r)
             if r['status'] != m_const.ROUTER_STATUS_ACTIVE:
                 data = {'router': {'status': m_const.ROUTER_STATUS_ACTIVE}}
-                r = super(MidonetL3ServicePlugin,
-                        self).update_router(context, id, data)
+                r = super(MidonetL3ServicePlugin, self).update_router(
+                    context, id, data)
         except Exception as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to update a router %(r_id)s in MidoNet: "
-                              "%(err)s"), {"r_id": id, "err": ex})
+                LOG.error("Failed to update a router %(r_id)s in MidoNet: "
+                          "%(err)s", {"r_id": id, "err": ex})
                 try:
                     data = {'router': {'status': m_const.ROUTER_STATUS_ERROR}}
-                    super(MidonetL3ServicePlugin,
-                        self).update_router(context, id, data)
+                    super(MidonetL3ServicePlugin, self).update_router(
+                        context, id, data)
                 except Exception:
-                    LOG.exception(_LE("Failed to update a router "
-                                      "status %s"), id)
+                    LOG.exception("Failed to update a router status %s", id)
         return r
 
     @log_helpers.log_method_call
@@ -163,7 +160,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     def delete_router(self, context, id):
         self._check_router_not_in_use(context, id)
 
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call delete_port inside
             # of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -176,7 +173,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @db_api.retry_if_session_inactive()
     def add_router_interface(self, context, router_id, interface_info):
         by_port = bool(interface_info.get('port_id'))
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call create_port/update_port
             # inside of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -189,9 +186,9 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         try:
             self.client.add_router_interface_postcommit(router_id, info)
         except Exception as ex:
-            LOG.error(_LE("Failed to create MidoNet resources to add router "
-                          "interface. info=%(info)s, router_id=%(router_id)s, "
-                          "error=%(err)r"),
+            LOG.error("Failed to create MidoNet resources to add router "
+                      "interface. info=%(info)s, router_id=%(router_id)s, "
+                      "error=%(err)r",
                       {"info": info, "router_id": router_id, "err": ex})
             with excutils.save_and_reraise_exception():
                 if not by_port:
@@ -202,7 +199,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def remove_router_interface(self, context, router_id, interface_info):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call delete_port inside
             # of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -217,7 +214,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def create_floatingip(self, context, floatingip):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call create_port inside
             # of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -229,19 +226,19 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
             self.client.create_floatingip_postcommit(fip)
         except Exception as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to create floating ip %(fip)s: %(err)s"),
+                LOG.error("Failed to create floating ip %(fip)s: %(err)s",
                           {"fip": fip, "err": ex})
                 try:
                     self.delete_floatingip(context, fip['id'])
                 except Exception:
-                    LOG.exception(_LE("Failed to delete a floating ip %s"),
+                    LOG.exception("Failed to delete a floating ip %s",
                                   fip['id'])
         return fip
 
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def delete_floatingip(self, context, id):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             # REVISIT(yamamoto): This should not call delete_port inside
             # of a transaction.
             setattr(context, 'GUARD_TRANSACTION', False)
@@ -253,7 +250,7 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
     @log_helpers.log_method_call
     @db_api.retry_if_session_inactive()
     def update_floatingip(self, context, id, floatingip):
-        with context.session.begin(subtransactions=True):
+        with db_api.context_manager.writer.using(context):
             fip = super(MidonetL3ServicePlugin, self).update_floatingip(
                 context, id, floatingip)
             self.client.update_floatingip_precommit(context, id, fip)
@@ -269,17 +266,18 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
             self.client.update_floatingip_postcommit(id, fip)
         except Exception as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Failed to update a floating ip "
-                              "%(fip_id)s in MidoNet: "
-                              "%(err)s"), {"fip_id": id, "err": ex})
+                LOG.error("Failed to update a floating ip "
+                          "%(fip_id)s in MidoNet: %(err)s",
+                          {"fip_id": id, "err": ex})
                 try:
                     self.update_floatingip_status(
-                            context, id, n_const.FLOATINGIP_STATUS_ERROR)
+                        context, id, n_const.FLOATINGIP_STATUS_ERROR)
                 except Exception:
-                    LOG.exception(_LE("Failed to update floating ip "
-                                      "status %s"), id)
+                    LOG.exception("Failed to update floating ip status %s",
+                                  id)
         return fip
 
+    @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_DELETE])
     def _check_router_interface_used_as_gw_for_fip(self, resource,
                                                    event, trigger, **kwargs):
         context = kwargs['context']
@@ -288,9 +286,3 @@ class MidonetL3ServicePlugin(common_db_mixin.CommonDbMixin,
         if self._subnet_has_fip(context, router_id, subnet_id):
             raise routerinterfacefip.RouterInterfaceInUseAsGatewayByFloatingIP(
                 router_id=router_id, subnet_id=subnet_id)
-
-    def __subscribe(self):
-        registry.subscribe(
-            self._check_router_interface_used_as_gw_for_fip,
-            resources.ROUTER_INTERFACE,
-            events.BEFORE_DELETE)
